#include "ili93xx.h"

#define ILI9326_PANEL	PANEL_CPT28

static inline void ILI9326_Poweron()
{
#if (ILI9326_PANEL == PANEL_CPT28)
	ILI93XX_CtrlWrite(0x0100, 0x1190); // SAP, BT[3:0], AP, DSTB, SLP, STB
	ILI93XX_CtrlWrite(0x0101, 0x0227); // DC1 [2:0], DC0 [2:0], VC [2:0]
	mdelay (50); 
	ILI93XX_CtrlWrite(0x0102, 0x01BD); // VREG1OUT voltage
	mdelay (50); 
	ILI93XX_CtrlWrite(0x0103, 0x2D00); // VDV [4:0] for VCOM amplitude 
	ILI93XX_CtrlWrite(0x0281, 0x000E); // VCM [5:0] for VCOMH 
#elif (ILI9326_PANEL == PANEL_WINTEK28)
	ILI93XX_CtrlWrite(0x0100, 0x1190); // SAP, BT[3:0], AP, DSTB, SLP, STB
	ILI93XX_CtrlWrite(0x0101, 0x0227); // DC1 [2:0], DC0 [2:0], VC [2:0]
	mdelay (50); 
	ILI93XX_CtrlWrite(0x0102, 0x01BD); // VREG1OUT voltage
	mdelay (50); 
	ILI93XX_CtrlWrite(0x0103, 0x2F00); // VDV [4:0] for VCOM amplitude 
	ILI93XX_CtrlWrite(0x0281, 0x0008); // VCM [5:0] for VCOMH 
#elif (ILI9326_PANEL == PANEL_LPL30)
	ILI93XX_CtrlWrite(0x0100, 0x1190); // SAP, BT[3:0], AP, DSTB, SLP, STB
	ILI93XX_CtrlWrite(0x0101, 0x0127); // DC1 [2:0], DC0 [2:0], VC [2:0]
	mdelay (50); 
	ILI93XX_CtrlWrite(0x0102, 0x01B8); // VREG1OUT voltage
	mdelay (50); 
	ILI93XX_CtrlWrite(0x0103, 0x3A00); // VDV [4:0] for VCOM amplitude 
	ILI93XX_CtrlWrite(0x0281, 0x0006); // VCM [5:0] for VCOMH 
#endif
	mdelay (50); 
}

static inline void ILI9326_Gamma()
{
#if (ILI9326_PANEL == PANEL_CPT28)
	ILI93XX_CtrlWrite(0x0300, 0x0000);
	ILI93XX_CtrlWrite(0x0301, 0x0707);
	ILI93XX_CtrlWrite(0x0302, 0x0606);
	ILI93XX_CtrlWrite(0x0305, 0x0000);
	ILI93XX_CtrlWrite(0x0306, 0x0D00);
	ILI93XX_CtrlWrite(0x0307, 0x0706);
	ILI93XX_CtrlWrite(0x0308, 0x0005);
	ILI93XX_CtrlWrite(0x0309, 0x0007);
	ILI93XX_CtrlWrite(0x030C, 0x0000);
	ILI93XX_CtrlWrite(0x030D, 0x000A);
#elif (ILI9326_PANEL == PANEL_WINTEK28)
	ILI93XX_CtrlWrite(0x0300, 0x0000);
	ILI93XX_CtrlWrite(0x0301, 0x0406);
	ILI93XX_CtrlWrite(0x0302, 0x0303);
	ILI93XX_CtrlWrite(0x0305, 0x0005);
	ILI93XX_CtrlWrite(0x0306, 0x1604);
	ILI93XX_CtrlWrite(0x0307, 0x0404);
	ILI93XX_CtrlWrite(0x0308, 0x0103);
	ILI93XX_CtrlWrite(0x0309, 0x0007);
	ILI93XX_CtrlWrite(0x030C, 0x0700);
	ILI93XX_CtrlWrite(0x030D, 0x0008);
#elif (ILI9326_PANEL == PANEL_LPL30)
	ILI93XX_CtrlWrite(0x0300, 0x0000);
	ILI93XX_CtrlWrite(0x0301, 0x0506);
	ILI93XX_CtrlWrite(0x0302, 0x0405);
	ILI93XX_CtrlWrite(0x0305, 0x0007);
	ILI93XX_CtrlWrite(0x0306, 0x0008);
	ILI93XX_CtrlWrite(0x0307, 0x0203);
	ILI93XX_CtrlWrite(0x0308, 0x0102);
	ILI93XX_CtrlWrite(0x0309, 0x0007);
	ILI93XX_CtrlWrite(0x030C, 0x0500);
	ILI93XX_CtrlWrite(0x030D, 0x000F);
#endif
}

void ILI9326_Initial(void)
{
	// VCI=2.8V
	//************* Reset LCD Driver ****************//
	ILI93XX_Reset(1);
	mdelay(1);
	ILI93XX_Reset(0);
	mdelay(10); // This delay time is necessary
	ILI93XX_Reset(1);
	mdelay(50);
	//************* Start Initial Sequence **********//
	ILI93XX_CtrlWrite(0x0702, 0x3008); // Set internal timing
	ILI93XX_CtrlWrite(0x0705, 0x0036); // Set internal timing
	ILI93XX_CtrlWrite(0x070B, 0x1213); // Set internal timing
	ILI93XX_CtrlWrite(0x0001, 0x0100); // set SS and SM bit 
	ILI93XX_CtrlWrite(0x0002, 0x0100); // set 1 line inversion 
	ILI93XX_CtrlWrite(0x0003, 0x1030); // set GRAM write direction and BGR=1.
	ILI93XX_CtrlWrite(0x0008, 0x0202); // set the back porch and front porch
	ILI93XX_CtrlWrite(0x0009, 0x0000); // set non-display area refresh cycle ISC [3:0] 
	ILI93XX_CtrlWrite(0x000C, 0x0000); // RGB interface setting
	ILI93XX_CtrlWrite(0x000F, 0x0000); // RGB interface polarity

	//************* Power On sequence ****************//
	ILI93XX_CtrlWrite(0x0100, 0x0000); // SAP, BT [3:0], AP, DSTB, SLP, STB
	ILI93XX_CtrlWrite(0x0102, 0x0000); // VREG1OUT voltage 
	ILI93XX_CtrlWrite(0x0103, 0x0000); // VDV[4:0] for VCOM amplitude
	mdelay (200);                         // Dis-charge capacitor power voltage
	ILI9326_Poweron();

	ILI93XX_CtrlWrite(0x0200, 0x0000); // GRAM horizontal Address
	ILI93XX_CtrlWrite(0x0201, 0x0000); // GRAM Vertical Address
	// ----------- Adjust the Gamma Curve ----------//
	ILI9326_Gamma();
	//------------------ Set GRAM area ---------------//
	ILI93XX_CtrlWrite(0x0210, 0x0000); // Horizontal GRAM Start Address 
	ILI93XX_CtrlWrite(0x0211, 0x00EF); // Horizontal GRAM End Address
	ILI93XX_CtrlWrite(0x0212, 0x0000); // Vertical GRAM Start Address
	ILI93XX_CtrlWrite(0x0213, 0x01AF); // Vertical GRAM Start Address
#if (ILI9326_PANEL == PANEL_CPT28)
	ILI93XX_CtrlWrite(0x0400, 0x3100); // Gate Scan Line 400 lines
#else
	ILI93XX_CtrlWrite(0x0400, 0x3500); // Gate Scan Line 400 lines
#endif
	ILI93XX_CtrlWrite(0x0401, 0x0001); // NDL,VLE, REV
	ILI93XX_CtrlWrite(0x0404, 0x0000); // set scrolling line
	//-------------- Partial Display Control ---------//
	ILI93XX_CtrlWrite(0x0500, 0x0000); //Partial Image 1 Display Position
	ILI93XX_CtrlWrite(0x0501, 0x0000); //Partial Image 1 RAM Start/End Address
	ILI93XX_CtrlWrite(0x0502, 0x0000); //Partial Image 1 RAM Start/End Address
	ILI93XX_CtrlWrite(0x0503, 0x0000); //Partial Image 2 Display Position
	ILI93XX_CtrlWrite(0x0504, 0x0000); //Partial Image 2 RAM Start/End Address
	ILI93XX_CtrlWrite(0x0505, 0x0000); //Partial Image 2 RAM Start/End Address
	//-------------- Panel Control -------------------//
	ILI93XX_CtrlWrite(0x0010, 0x0010); //DIVI[1:0];RTNI[4:0]
	ILI93XX_CtrlWrite(0x0011, 0x0600); //NOWI[2:0];SDTI[2:0]
	ILI93XX_CtrlWrite(0x0020, 0x0002); //DIVE[1:0];RTNE[5:0]
	ILI93XX_CtrlWrite(0x0007, 0x0173); // 262K color and display ON
}

void ILI9326_ExitSleep(void)
{
	//************* Power On sequence ******************//
	ILI93XX_CtrlWrite(0x0100, 0x0000); // SAP, BT[3:0], AP, DSTB, SLP
	mdelay (200); // Dis-charge capacitor power voltage
	ILI9326_Poweron();
	ILI93XX_CtrlWrite(0x0007, 0x0173); // 262K color and display ON
}

void ILI9326_EnterSleep(void)
{
	ILI93XX_CtrlWrite(0x0007, 0x0131); // Set D1=0, D0=1
	mdelay (10);
	ILI93XX_CtrlWrite(0x0007, 0x0130); // Set D1=0, D0=0
	mdelay (10); 
	ILI93XX_CtrlWrite(0x0007, 0x0000); // display OFF
	//************* Power OFF sequence **************//
	ILI93XX_CtrlWrite(0x0100, 0x0000); // SAP, BT[3:0], APE, AP, DSTB, SLP
	mdelay (200);                     // Dis-charge capacitor power voltage
	ILI93XX_CtrlWrite(0x0100, 0x0002); // SAP, BT[3:0], APE, AP, DSTB, SLP
}
